//+------------------------------------------------------------------+
//|                                               Bands for Marc.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 LightSeaGreen
#property indicator_color2 LightSeaGreen
#property indicator_color3 LightSeaGreen
//---- indicator parameters
extern int    BandsPeriod=20;
extern int    MAPeriod_M1=20;
extern int    MAPeriod_M5=20;
extern int    MAPeriod_M15=20;
extern int    MAPeriod_M30=20;
extern int    MAPeriod_H1=20;
extern int    MAPeriod_H4=20;
extern int    MAPeriod_D1=20;
extern int    MAPeriod_W1=20;
extern int    BandsShift=0;
extern double BandsDeviations=2.0;
//---- buffers
double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,MovingBuffer);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,UpperBuffer);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,LowerBuffer);
//----
   SetIndexDrawBegin(0,BandsPeriod+BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Bollinger Bands                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int    i,k,counted_bars=IndicatorCounted();
   double deviation;
   double sum,oldval,newres;
//----
   if(Bars<=BandsPeriod) return(0);
//---- initial zero
   if(counted_bars<1)
      for(i=1;i<=BandsPeriod;i++)
        {
         MovingBuffer[Bars-i]=EMPTY_VALUE;
         UpperBuffer[Bars-i]=EMPTY_VALUE;
         LowerBuffer[Bars-i]=EMPTY_VALUE;
        }
//----
   int limit=Bars-counted_bars;
   if(counted_bars>0) limit++;
   for(i=0; i<limit; i++)
      MovingBuffer[i]=iMA(NULL,0,BandsPeriod,BandsShift,MODE_SMA,PRICE_CLOSE,i);
//----
   i=Bars-BandsPeriod+1;
   if(counted_bars>BandsPeriod-1) i=Bars-counted_bars-1;
   while(i>=0)
     {
      sum=0.0;
      k=i+BandsPeriod-1;
      oldval=MovingBuffer[i];
      while(k>=i)
        {
         newres=Close[k]-oldval;
         sum+=newres*newres;
         k--;
        }
      deviation=BandsDeviations*MathSqrt(sum/BandsPeriod);
      UpperBuffer[i]=oldval+deviation;
      LowerBuffer[i]=oldval-deviation;
      i--;
     }
     
    if (Period()!=PERIOD_M1)
    {
      ObjectCreate("M1",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_M1,MAPeriod_M1,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("M1",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_M1,MAPeriod_M1,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("M1",OBJPROP_COLOR,Red);
      else
         ObjectSet("M1",OBJPROP_COLOR,Lime);
    }
    if (Period()!=PERIOD_M5)
    {
      ObjectCreate("M5",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_M5,MAPeriod_M5,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("M5",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_M5,MAPeriod_M5,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("M5",OBJPROP_COLOR,OrangeRed);
      else
         ObjectSet("M5",OBJPROP_COLOR,MediumSpringGreen);
    }
    if (Period()!=PERIOD_M15)
    {
      ObjectCreate("M15",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_M15,MAPeriod_M15,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("M15",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_M15,MAPeriod_M15,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("M15",OBJPROP_COLOR,Tomato);
      else
         ObjectSet("M15",OBJPROP_COLOR,LimeGreen);
    }
    if (Period()!=PERIOD_M30)
    {
      ObjectCreate("M30",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_M30,MAPeriod_M30,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("M30",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_M30,MAPeriod_M30,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("M30",OBJPROP_COLOR,DeepPink);
      else
         ObjectSet("M30",OBJPROP_COLOR,GreenYellow);
    }
    if (Period()!=PERIOD_H1)
    {
      ObjectCreate("H1",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_H1,MAPeriod_H1,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("H1",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_H1,MAPeriod_H1,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("H1",OBJPROP_COLOR,Crimson);
      else
         ObjectSet("H1",OBJPROP_COLOR,Aquamarine);
    }
    if (Period()!=PERIOD_H4)
    {
      ObjectCreate("H4",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_H4,MAPeriod_H4,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("H4",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_H4,MAPeriod_H4,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("H4",OBJPROP_COLOR,Maroon);
      else
         ObjectSet("H4",OBJPROP_COLOR,LawnGreen);
    }
    if (Period()!=PERIOD_D1)
    {
      ObjectCreate("D1",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_D1,MAPeriod_D1,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("D1",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_D1,MAPeriod_D1,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("D1",OBJPROP_COLOR,IndianRed);
      else
         ObjectSet("D1",OBJPROP_COLOR,MediumSpringGreen);
    }
    if (Period()!=PERIOD_W1)
    {
      ObjectCreate("W1",OBJ_HLINE,0,0,iMA(Symbol(),PERIOD_D1,MAPeriod_W1,0,MODE_SMA,PRICE_CLOSE,0));
      ObjectSet("W1",OBJPROP_STYLE,STYLE_DASH);
      if(iMA(Symbol(),PERIOD_W1,MAPeriod_W1,0,MODE_SMA,PRICE_CLOSE,0)<iMA(NULL,0,BandsPeriod,0,MODE_SMA,PRICE_CLOSE,0))
         ObjectSet("W1",OBJPROP_COLOR,Brown);
      else
         ObjectSet("W1",OBJPROP_COLOR,PaleGreen);
    }
    
    
   return(0);
  }
//+------------------------------------------------------------------+
int deinit()
{
   ObjectsDeleteAll();
}